@@ -25,7 +25,7 @@ _Base = declarative_base()
 
 _GET = sqltext("""\
 select
-    uid, node
+    uid, node, accepted_conditions
 from
     user_nodes
 where
@@ -37,15 +37,22 @@ and
 
 _INSERT = sqltext("""\
 insert into user_nodes
-    (service, email, node)
+    (service, email, node, accepted_conditions)
 values
-    (:service, :email, :node)
+    (:service, :email, :node, :accepted_conditions)
 """)
 
 
 WRITEABLE_FIELDS = ['available', 'current_load', 'capacity', 'downed',
                     'backoff']
 
+_INSERT_METADATA = sqltext("""\
+insert into metadata
+    (service, name, value, needs_acceptance)
+values
+    (:service, :name, :value, :needs_acceptance)
+""")
+
 
 class SQLMetadata(object):
 
@@ -79,8 +86,10 @@ class SQLMetadata(object):
         self.user_nodes = get_cls('user_nodes', _Base)
         self.nodes = get_cls('nodes', _Base)
         self.patterns = get_cls('service_pattern', _Base)
+        self.metadata = get_cls('metadata', _Base)
 
-        for table in [self.user_nodes, self.nodes, self.patterns]:
+        for table in (self.user_nodes, self.nodes, self.patterns,
+                      self.metadata):
             table.metadata.bind = self._engine
             if create_tables:
                 table.create(checkfirst=True)
@@ -113,17 +122,38 @@ class SQLMetadata(object):
     # Node allocation
     #
     def get_node(self, email, service):
+        """Return the node of an user for a particular service. If the user
+        isn't assigned to a node yet, return None.
+
+        In addition to the node, this method returns the uid of the user and
+        a list of urls that the user needs to accept: (uid, node, to_accept)
+
+        - If the user isn't know. idx and node are set to None.
+        - If the user is known, idx and node are her id and assigned node.
+        - If the user didn't accepted everything, to_accept contains a list of
+          urls to read and accept.
+        - If the user agreed to the preconditions, to_accept is set to None
+        """
         res = self._safe_execute(_GET, email=email, service=service)
         try:
             one = res.fetchone()
+            if one is None or one.accepted_conditions == 0:
+                to_accept = [i.value for i in self.get_metadata(
+                             service, needs_acceptance=True)]
+
+                if not to_accept:
+                    to_accept = None
+            else:
+                to_accept = None
+
             if one is None:
-                return None, None
-            return one.uid, one.node
+                return None, None, to_accept
+            return one.uid, one.node, to_accept
         finally:
             res.close()
 
     def allocate_node(self, email, service):
-        uid, node = self.get_node(email, service)
+        uid, node, _ = self.get_node(email, service)
         if (uid, node) != (None, None):
             return uid, node
 
@@ -135,7 +165,8 @@ class SQLMetadata(object):
             res = self._safe_execute(_INSERT, email=email, service=service,
                                      node=node)
         except IntegrityError:
-            return self.get_node(email, service)
+            uid, node, _ = self.get_node(email, service)
+            return uid, node
 
         lastrowid = res.lastrowid
         res.close()
@@ -154,9 +185,6 @@ class SQLMetadata(object):
         res.close()
         return patterns
 
-    def _get_nodes_table(self, service):
-        return self.nodes
-
     def get_best_node(self, service):
         """Returns the 'least loaded' node currently available, increments the
         active count on that node, and decrements the slots currently available
@@ -191,3 +219,73 @@ class SQLMetadata(object):
         con.close()
 
         return node
+
+    def _get_metadata_table(self, service):
+        return self.metadata
+
+    def _get_nodes_table(self, service):
+        return self.nodes
+
+    def _get_user_nodes_table(self, service):
+        return self.user_nodes
+
+    def set_metadata(self, service, name, value, needs_acceptance=False):
+        def _insert():
+            self._safe_execute(_INSERT_METADATA, service=service, name=name,
+                               value=value, needs_acceptance=needs_acceptance,
+                               close=True)
+
+        def _update():
+
+            where = [metadata.c.service == service, metadata.c.name == name]
+            where = and_(*where)
+
+            fields = {'value': value, 'needs_acceptance': needs_acceptance}
+            query = update(metadata, where, fields)
+            self._safe_execute(query, close=True)
+
+        # first do a request to check if the metadata record exists or not
+        metadata = self._get_metadata_table(service)
+        where = [metadata.c.service == service, metadata.c.name == name]
+        query = select([metadata.c.value]).where(and_(*where))
+        res = self._safe_execute(query, close=True)
+        if res.fetchone() is None:
+            return _insert()
+        else:
+            return _update()
+
+    def get_metadata(self, service, name=None, needs_acceptance=None):
+        metadata = self._get_metadata_table(service)
+        where = [metadata.c.service == service]
+
+        if name is not None:
+            where.append(metadata.c.name == name)
+
+        if needs_acceptance is not None:
+            where.append(metadata.c.needs_acceptance == needs_acceptance)
+
+        fields = [metadata.c.name, metadata.c.value,
+                  metadata.c.needs_acceptance]
+
+        query = select(fields).where(and_(*where))
+        res = self._safe_execute(query)
+
+        return res.fetchall()
+
+    def set_accepted_conditions_flag(self, service, value, email=None):
+        """Update the 'conditions accepted' flag for a service.
+
+        If email is set to None, update the flag for all the users of this
+        service.
+        """
+        user_nodes = self._get_user_nodes_table(service)
+
+        where = [user_nodes.c.service == service, ]
+        if email is not None:
+            where.append(user_nodes.c.email == email)
+
+        where = and_(*where)
+
+        fields = {'accepted_conditions': value}
+        query = update(user_nodes, where, fields)
+        self._safe_execute(query, close=True)
